<?php

if(file_exists('0.3.1.db')){
    echo 'file exists';
    return false;
}

function normalize_kana($str)
{
    return mb_convert_kana($str, 'KVrn', 'UTF-8');
}

function normalize_time($str)
{
    return $str;
}

function is_valid_date_string($s)
{
    if($s === ''){
        return true;
    }

    if(!preg_match('/^[0-9]{4}\/[0-9]{2}\/[0-9]{2}$/', $s)){
        return false;
    }

    list($y, $m, $d) = explode('/', $s);

    if(!checkdate($m, $d, $y)){
        return false;
    }
    
    return true;
}

function is_valid_time_string($s)
{
    if($s === ''){
        return true;
    }

    if(!preg_match('/^\d+(?:\.\d+)?$/', $s)){
        return false;
    }

    $time = (double)$s;

    if(($time != 0) && (fmod($time, 0.25) != 0)){
        return false;
    }

    return true;
}

function db_create_schema($dbh)
{
    $sql = <<<EOS
CREATE TABLE tasks(
id                       INTEGER,
revision_id              INTEGER DEFAULT 0,
accept_date              INTEGER,
accept_person            TEXT,
applicant                TEXT,
content                  TEXT,
deleted                  INTEGER DEFAULT 0,
update_date              INTEGER,
remote_addr              TEXT,
PRIMARY KEY(id, revision_id)
)
EOS;

    $res = $dbh->exec($sql);
    if(!$res){
        // print_r($dbh->errorInfo());
    }

    $sql = <<<EOS
CREATE TABLE tickets(
id                       INTEGER,
revision_id              INTEGER DEFAULT 0,
task_id                  INTEGER,
person                   TEXT,
content                  TEXT,
estimate_time            REAL,
results_time             REAL,
completion_date          INTEGER,
deleted                  INTEGER DEFAULT 0,
update_date              INTEGER,
remote_addr              TEXT,
PRIMARY KEY(id, revision_id)
)
EOS;

    $res = $dbh->exec($sql);
    if(!$res){
        // print_r($dbh->errorInfo());
    }

    $sql = <<<EOS
CREATE VIEW valid_tasks AS
SELECT
 *
FROM
 (
 SELECT
  t1.id AS id,
  MAX(t2.revision_id) AS revision_id
 FROM
  tasks t1
  LEFT JOIN tasks t2
  ON (t1.id=t2.id)
 GROUP BY
  t1.id
 ) AS t3
 LEFT JOIN tasks t
 ON t.id=t3.id AND t.revision_id=t3.revision_id

WHERE
 deleted=0
EOS;

    $res = $dbh->exec($sql);
    if(!$res){
        // print_r($dbh->errorInfo());
    }

    $sql = <<<EOS
CREATE VIEW valid_tickets AS
SELECT
 t.*
FROM
 (
 SELECT
  t1.id AS id,
  MAX(t2.revision_id) AS revision_id
 FROM
  tickets t1
  LEFT JOIN tickets t2
  ON (t1.id=t2.id)
 GROUP BY
  t1.id
 ) AS t3
 LEFT JOIN tickets t
  ON t.id=t3.id AND t.revision_id=t3.revision_id
 JOIN valid_tasks tasks
  ON t.task_id=tasks.id

WHERE
 t.deleted=0 AND tasks.deleted=0
EOS;

    $dbh->exec($sql);

}

$dsn_old = 'sqlite:' . dirname(__FILE__) . '/0.3.db';
$dsn_new = 'sqlite:' . dirname(__FILE__) . '/0.3.1.db';

$dbh_old = new PDO($dsn_old);
$dbh_new = new PDO($dsn_new);

// e[u̍쐬

db_create_schema($dbh_new);

// taskse[üڍs

$sql = 'SELECT * FROM tasks';
$sth = $dbh_old->prepare($sql);
if($sth === false){
    echo 'prepare error';
    return false;
}

$res = $sth->execute();
if($res === false){
    echo 'execute error';
    return false;
}

while(($task = $sth->fetch(PDO::FETCH_ASSOC)) !== false){
    $sql_ins = <<<EOS
INSERT INTO tasks
 (
 id,
 revision_id,
 accept_date,
 accept_person,
 applicant,
 content,
 deleted, 
 update_date,
 remote_addr) 
VALUES
 (
 :id,
 :revision_id,
 :accept_date,
 :accept_person,
 :applicant,
 :content,
 :deleted,
 :update_date,
 :remote_addr
 )
EOS;

    $sth_ins = $dbh_new->prepare($sql_ins);
    if($sth_ins === false){
        print_r( $dbh_new->errorInfo() );
        exit;
    }

    if(!is_valid_date_string($task['accept_date'])){
        echo <<<EOS
<a href="index.php?a=tv&amp;id={$task['id']}">{$task['id']}</a> tG[ł {$task['accept_date']}<br />

EOS;
    }

    $params = array(
        ':id' => $task['id'],
        ':revision_id' => $task['revision_id'],
        ':accept_date' => $task['accept_date'],
        ':accept_person' => normalize_kana($task['accept_person']),
        ':applicant' => normalize_kana($task['applicant']),
        ':content' => normalize_kana($task['content']),
        ':deleted' => normalize_kana($task['deleted']),
        ':remote_addr' => normalize_kana($task['remote_addr']),
        ':update_date' => $task['update_date']);

    $res_ins = $sth_ins->execute($params);
    if($res_ins === false){
        echo "{$task['id']}, {$task['revision_id']}\n";
        print_r( $dbh_new->errorInfo() );
        exit;
    }
}


// ticketse[üڍs

$sql = 'SELECT * FROM tickets';
$sth = $dbh_old->prepare($sql);
if($sth === false){
    echo 'prepare error';
    return false;
}

$res = $sth->execute();
if($res === false){
    echo 'execute error';
    return false;
}

while(($ticket = $sth->fetch(PDO::FETCH_ASSOC)) !== false){
    $sql_ins = <<<EOS
INSERT INTO tickets(
 id,
 revision_id,
 task_id,
 person,
 content,
 estimate_time,
 results_time,
 completion_date,
 deleted,
 update_date,
 remote_addr)
VALUES (
 :id,
 :revision_id,
 :task_id,
 :person,
 :content,
 :estimate_time,
 :results_time,
 :completion_date,
 :deleted,
 :update_date,
 :remote_addr)
EOS;

    $sth_ins = $dbh_new->prepare($sql_ins);
    if($sth_ins === false){
        print_r( $dbh_new->errorInfo() );
        exit;
    }

    if(!is_valid_date_string($ticket['completion_date'])){
        echo <<<EOS
<a href="index.php?a=tie&amp;id={$ticket['id']}">{$ticket['id']}</a> tG[ł {$ticket['completion_date']}<br />

EOS;
    }

    if(!is_valid_time_string($ticket['estimate_time'])){
        echo <<<EOS
<a href="index.php?a=tie&amp;id={$ticket['id']}">{$ticket['id']}</a> ώԃG[ł {$ticket['estimate_time']}<br />

EOS;
    }

    if(!is_valid_time_string($ticket['results_time'])){
        echo <<<EOS
<a href="index.php?a=tie&amp;id={$ticket['id']}">{$ticket['id']}</a> ώԃG[ł {$ticket['results_time']}<br />

EOS;
    }

    $params = array(
        ':id' => $ticket['id'],
        ':revision_id' => $ticket['revision_id'],
        ':task_id' => $ticket['task_id'],
        ':person' => normalize_kana($ticket['person']),
        ':content' => normalize_kana($ticket['content']),
        ':estimate_time' => normalize_time($ticket['estimate_time']),
        ':results_time' => normalize_time($ticket['results_time']),
        ':completion_date' => $ticket['completion_date'],
        ':deleted' => $ticket['deleted'],
        ':update_date' => $ticket['update_date'],
        ':remote_addr' => $ticket['remote_addr']
        );

    $res_ins = $sth_ins->execute($params);
    if($res_ins === false){
        print_r( $dbh->errorInfo() );
        exit;
    }
}
