<?php

/*
 * Copyright (c) 2009 Hajime Miyauchi
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 *
 * $Id$
 */

date_default_timezone_set('Asia/Tokyo');
error_reporting(E_ALL | E_STRICT);
mb_internal_encoding('UTF-8');
mb_language('ja');
mb_regex_encoding('UTF-8');

$dsn = 'sqlite:' . dirname(__FILE__) . '/0.3.1.db';


/// HTML関連の関数群

function h($str)
{
    return htmlspecialchars($str, ENT_QUOTES, 'UTF-8');
}

function ue($str)
{
    return rawurlencode($str);
}

function make_summary($text, $len)
{
    $t = mb_strimwidth($text, 0, $len, '...', mb_internal_encoding());
    $t = preg_replace('/\n/', '', $t);

    return $t;
}

function normalize_kana($str)
{
    return mb_convert_kana($str, 'KVrn', 'UTF-8');
}

/**
 * @param $text 0000/00/00
 */
function text_to_date($text)
{
    $ts = strtotime($text);
    if($ts === FALSE){
        return '';
    }
    
    $t = short_date($ts);
    
    return $t;
}

function short_date($date)
{
    $jweek = array('日',
                   '月',
                   '火',
                   '水',
                   '木',
                   '金',
                   '土',
                   '日');
    
    $t = '';
    if(date('Y') != date('Y', $date)){
        $t .= date('Y年', $date);
    }

    $t .= date('n月j日', $date);

    $t .= '(' . $jweek[date('w', $date)] . ')';

    return $t;
}

function convert_text_to_html($text, $max_line = 0)
{
    $lines = explode("\n", $text);
    
    if($max_line > 0){
        if(count($lines) > $max_line){
            $lines[$max_line - 1] .= '...';
        }

        $lines = array_splice($lines, 0, $max_line);
    }

    $text = implode("\n", $lines);

    $h = preg_replace('/\n/', '<br/>', h($text));
    $h = preg_replace('/ /', '&nbsp;', $h);

    $h = preg_replace('/(https?:\/\/[-_.!~*\'()a-zA-Z0-9;\/?:@&=+$,%#]+)/',
                         '<a href="${1}">${1}</a>',
                         $h);

    return $h;
}

function make_html($title, $body)
{
    $h = '';
    $h .= '<html>' . "\n";
    $h .= '<head>' . "\n";
    $h .= '<meta http-equiv="Content-Type" content="text/html; charset=utf-8" />' . "\n";
    $h .= '<link href="main.css" rel="stylesheet" type="text/css" />' . "\n";
    $h .= '<title>' . h($title) . '</title>' . "\n";
    $h .= '<script src="main.js"></script>';
    $h .= '</head>' . "\n";
    $h .= '<body>';
    $h .= '<p>';
    $h .= '<a href="index.php">HOME</a> | ';
    $h .= '<a href="index.php?a=tn">タスクの追加</a> | ';
    $h .= '<a href="index.php?a=search">検索</a> | ';
    $h .= '<a href="index.php?a=todo">ToDo</a> ';
    $h .= '</p>';
    $h .= $body;
    $h .= '<div style="text-align: right">';
    $h .= '<a href="index.php?a=rss">RSS</a>';
    $h .= '<p>SimpleTask 2.0</p>';
    $h .= '</div>';
    $h .= '</body>';
    $h .= '</html>';

    return $h;
}

function make_html_task_list($tasks, $short = false)
{
    $h = '';

    $h .= '<table style="width: 100%">';
    $h .= '<tr>';
    $h .= '<th style="width: 5em">ID</th>';
    $h .= '<th>申告者</th>';
    $h .= '<th>内容</th>';
    $h .= '<th>受付者</th>';
    $h .= '<th>作業日</th>';
    $h .= '<th>更新日</th>';
    if(!$short){
        $h .= '<th>チケット</th>';
        $h .= '<th>未完</th>';
        $h .= '<th>未完時間</th>';
        $h .= '<th>実績時間</th>';
    }
    
    $h .= '</tr>';

    $h .= "\n";

    foreach($tasks as $task){
        if(trim($task['incomplete_ticket_count']) > 0 || $task['ticket_count'] == 0){
            $h .= '<tr class="open">';
        }else{
            $h .= '<tr class="close">';
        }
        
        $h .= '<td style="text-align:center">';
        $h .= '<a href="index.php?a=tv&amp;id=' . h($task['id']) . '" class="id">';
        $h .= h($task['id']);
        $h .= '</a>';
        $h .= '</td>';
        $h .= '<td>' . h($task['applicant']) . '</td>';
        $h .= '<td>' . convert_text_to_html(make_summary($task['content'], 40)) . '</td>';
        $h .= '<td>' . h($task['accept_person']) . '</td>';
        $h .= '<td class="right">' . h(text_to_date($task['completion_date2'])) . '</td>';
        $h .= '<td class="right">' . h(short_date($task['update_date2'])) . '</td>';
        if(!$short){
            $h .= '<td class="right">' . h($task['ticket_count']) . '</td>';
            $h .= '<td class="right">' . h($task['incomplete_ticket_count']) . '</td>';
            $h .= '<td class="right">' . h(sprintf('%.2f', $task['incomplete_time'])) . '</td>';
            $h .= '<td class="right">' . h(sprintf('%.2f', $task['total_results_time'])) . '</td>';
        }

        $h .= '</tr>';
        $h .= "\n";
    }

    $h .= '</table>';

    return $h;
}

function make_html_task_list_navigation($query, $limit, $offset, $total, $order)
{
    $h = '';
    $h .= '<p>';

    if($offset > 0 && $offset <= $total){
        // 戻るボタン
        $h .= '<a href="index.php?a=search&amp;q=';
        $h .= ue($query['text']);
        $h .= '&amp;of=';
        $h .= ue($offset - $limit);
        $h .= '&amp;p=';
        $h .= ue($query['person']);
        $h .= '&amp;d1=';
        $h .= ue($query['date1']);
        $h .= '&amp;d2=';
        $h .= ue($query['date2']);
        if($query['only_incomplete']){
            $h .= '&amp;oi';
        }

        $h .= '&amp;order=';

        $h .= ue($order);
        
        $h .= '">&lt;前</a>&nbsp;';
    }else{
        $h .= '<span class="close">&lt;前&nbsp;</span>';
    }

    if(($offset + $limit) < $total){
        // 進むボタン
        $h .= '<a href="index.php?a=search&amp;q=';
        $h .= ue($query['text']);
        $h .= '&amp;of=';
        $h .= h($offset + $limit);
        $h .= '&amp;p=';
        $h .= ue($query['person']);
        $h .= '&amp;d1=';
        $h .= ue($query['date1']);
        $h .= '&amp;d2=';
        $h .= ue($query['date2']);
        if($query['only_incomplete']){
            $h .= '&amp;oi';
        }
        
        $h .= '&amp;order=';

        $h .= ue($order);
        
        $h .= '">次&gt;</a>';
    }else{
        $h .= '<span class="close">次&gt;</span>';
    }

    $h .= '</p>';

    return $h;
}

/**
 * タスク編集画面
 */
function make_html_task_edit($task, $token, $view_mode = false)
{
    $h = '';

    if(is_null($task['id'])){
        $h .= '<form action="index.php?a=ta" method="post">';
    }else{
        $h .= '<form action="index.php?a=tu" method="post">';
    }

    $h .= '<input type="hidden" name="token" value="';
    $h .= h($token);
    $h .= '" />';    

    $h .= '<table style="width: 100%">';

    $h .= '<tr>';
    $h .= '<th style="width: 7em">ID(rev)</th>';
    $h .= '<td>';
    if($view_mode){
        $h .= h($task['id']) . '(' . h($task['revision_id']) . ')';
    }else{
        $h .= '<input type="text" value="' . h($task['id']) . '" size="10" disabled="disabled" />';
        $h .= '<input type="hidden" name="id" value="' . h($task['id']) . '" />';
    }
    
    $h .= '</td>';
    $h .= '</tr>';

    $h .= '<tr>';
    $h .= '<th>受付日時</th>';
    $h .= '<td>';
    if($view_mode){
        $h .= h(text_to_date($task['accept_date']));
    }else{
        $h .= '<input type="text" name="accept_date" value="' . h($task['accept_date']) . '" size="10" />';
        $h .= '<input type="button" value="今日" onclick="current_date(this.form.accept_date);" />';
        $h .= ' 例:2009/01/15';
    }

    $h .= '</td>';
    $h .= '</tr>';

    $h .= '<tr>';
    $h .= '<th>受付者</th>';
    $h .= '<td>';
    if($view_mode){
        $h .= h($task['accept_person']);
    }else{
        $h .= '<input type="text" name="accept_person" value="' . h($task['accept_person']) . '" size="10" />';
    }
    
    $h .= '</td>';
    $h .= '</tr>';

    $h .= '<tr>';
    $h .= '<th>申請者</th>';
    $h .= '<td>';
    if($view_mode){
        $h .= h($task['applicant']);
    }else{
        $h .= '<input type="text" name="applicant" value="' . h($task['applicant']) . '" size="30" />';
    }

    $h .= '</td>';

    $h .= '<tr>';
    $h .= '<th>内容</th>';
    $h .= '<td>';
    if($view_mode){
        $h .= convert_text_to_html($task['content']);
    }else{
        $h .= '<textarea name="content" rows="10" cols="80" class="content">';
        $h .= h($task['content']);
        $h .= '</textarea>';
    }
    
    $h .= '</td>';
    $h .= '</tr>';

    if($view_mode){
        $h .= '<tr>';
        $h .= '<th>更新元IP</th>';
        $h .= '<td>';
        $h .= $task['remote_addr'];
        $h .= '</td>';
        $h .= '</tr>';
        $h .= '<tr>';
        $h .= '<th>更新日</th>';
        $h .= '<td>';
        $h .= h(date('Y/m/d H:i', $task['update_date']));
        $h .= '</td>';
        $h .= '</tr>';
    }

    $h .= '</table>';

    if(!$view_mode){
        $h .= '<input type="submit" value="登録" />' . "\n";
    }
    
    $h .= '</form>';

    return $h;
}

function make_html_form_select_options($options, $selected_values = array())
{
    $h = '';

    foreach($selected_values as $v){
        $hit = false;
        
        foreach($options as $o){
            if($o['value'] == $v){
                $hit = true;
            }
        }

        if(!$hit){
            $option['value'] = $v;
            $option['text'] = $v;
            $options[] = $option;
        }
    }

    foreach($options as $option){
        $h .= '<option ';
        $h .= 'value="' . h($option['value']) . '" ';
        if(in_array($option['value'], $selected_values)){
            $h .= 'selected="selected" ';
        }

        $h .= '>';
        $h .= h($option['text']);
        $h .= '</option>';
    }

    return $h;
}

function make_html_task_search_form($query)
{
    $text = h($query['text']);

    $checked = '';
    if($query['only_incomplete']){
        $checked = ' checked="checked" ';
    }

    $person = h($query['person']);
    $date1 = h($query['date1']);
    $date2 = h($query['date2']);

    $h = <<<EOS
<form action="index.php" method="get">
 <input type="hidden" name="a" value="search" />
 <input type="text" name="q" size="20" value="{$text}" />
 <label>担当者 <input type="text" name="p" size="10" value="{$person}" /></label>
 <input type="text" name="d1" size="10" value="{$date1}" /> ～
 <input type="text" name="d2" size="10" value="{$date2}" />
 <label><input type="checkbox" name="oi" value="" {$checked} />未完タスクのみ</label>
 <select name="order">
 <option value="update_date">更新日順</option>
 <option value="completion_date">作業日順</option>
 </select>
 <input type="submit" value="検索" />
</form>
EOS;

    return $h;
}

function make_html_ticket_list($tickets)
{
    $h = '';
    $h .= '<table style="width: 100%">';
    $h .= '<tr>';
    $h .= '<th>ID</th>';
    $h .= '<th>担当</th>';
    $h .= '<th class="content">内容</th>';
    $h .= '<th>見積時間</th>';
    $h .= '<th>実績時間</th>';
    $h .= '<th>完了</th>';
    $h .= '<th>更新</th>';
    $h .= '</tr>';
    
    foreach($tickets as $ticket){
        if(trim($ticket['completion_date']) == ''){
            $h .= '<tr class="open">';
        }else{
            $h .= '<tr class="close">';
        }
        
        $h .= '<td>';
        $h .= '<a href="index.php?a=tie&amp;id=' . h($ticket['id']);
        $h .= '" class="id">';
        $h .= h($ticket['id']);
        $h .= '</a>';
        $h .= '</td>';
        
        $h .= '<td>';
        $h .= h($ticket['person']);
        $h .= '</td>';
        
        $h .= '<td class="content">';
        $h .= convert_text_to_html($ticket['content']);
        $h .= '</td>';
        
        $h .= '<td>';
        $h .= h($ticket['estimate_time']);
        $h .= '</td>';
        
        $h .= '<td>';
        $h .= h($ticket['results_time']);
        $h .= '</td>';
        
        $h .= '<td>';
        $h .= h(text_to_date($ticket['completion_date']));
        $h .= '</td>';
        
        $h .= '<td>';
        $h .= h(short_date($ticket['update_date']));
        $h .= '</td>';
        
        $h .= '</tr>' . "\n";
    }
    
    $h .= '</table>';
    
    return $h;
}

function make_html_ticket_edit($ticket, $token)
{
    $h = '';

    if(!is_null($ticket['id'])){
        $h .= '<p><a href="index.php?a=tv&amp;id=';
        $h .= h($ticket['task_id']);
        $h .= '">タスクへ</a></p>';
    }

    if(is_null($ticket['id'])){
        $h .= '<form action="index.php?a=tia" method="post">';
    }else{
        $h .= '<form action="index.php?a=tiu" method="post">';
    }

    $h .= '<input type="hidden" name="token" value="';
    $h .= h($token) . '" />';

    $h .= '<input type="hidden" name="task_id" value="';
    $h .= h($ticket['task_id']);
    $h .= '" />';

    $h .= '<input type="hidden" name="id" value="';
    $h .= h($ticket['id']);
    $h .= '" />';

    $h .= '<table>';

    $h .= '<tr>';
    $h .= '<th>ID</th>';
    $h .= '<td>';
    $h .= h($ticket['id']);
    $h .= '</td>';
    $h .= '</tr>';

    $h .= '<tr>';
    $h .= '<th>担当</th>';
    $h .= '<td>';
    $h .= '<input type="text" name="person" value="' . h($ticket['person']) . '" />';
    $h .= '</td>';
    $h .= '</tr>';

    $h .= '<tr>';
    $h .= '<th>内容</th>';
    $h .= '<td class="content">';
    $h .= '<textarea name="content" rows="10" cols="80">';
    $h .= h($ticket['content']);
    $h .= '</textarea>';
    $h .= '</td>';
    $h .= '</tr>';

    $h .= '<tr>';
    $h .= '<th>見積時間</th>';
    $h .= '<td>';
    $h .= '<input type="text" name="estimate_time" value="' . h($ticket['estimate_time']) . '" />';
    $h .= ' 例: 「0.25」(15分)、「1.0」(1時間)';
    $h .= '</td>';
    $h .= '</tr>';

    $h .= '<tr>';
    $h .= '<th>実績時間</th>';
    $h .= '<td>';
    $h .= '<input type="text" name="results_time" value="' . h($ticket['results_time']) . '" />';
    $h .= ' 例: 「0.25」(15分)、「1.0」(1時間)';
    $h .= '</td>';
    $h .= '</tr>';

    $h .= '<tr>';
    $h .= '<th>完了日</th>';
    $h .= '<td>';
    $h .= '<input type="text" name="completion_date" value="' . h($ticket['completion_date']) . '" />';
        $h .= '<input type="button" value="今日" onclick="current_date(this.form.completion_date);" />';
    $h .= ' 例: 「2009/01/15」';
    $h .= '</td>';
    $h .= '</tr>';

    $h .= '</table>';
    $h .= '<input type="submit" />';
    $h .= '</form>';

    return $h;
}

function make_html_todo($tickets)
{
    $h = '';
    $h .= '<p>TODO(人別)</p>';

    $h .= '<table width="100%">';
    $h .= '<tr>';
    $h .= '<th>ID</th>';
    $h .= '<th>タスクID</th>';
    $h .= '<th>担当</th>';
    $h .= '<th>申告者</th>';
    $h .= '<th>内容</th>';
    $h .= '<th>内容</th>';
    $h .= '<th>見積</th>';
    $h .= '<th>実績</th>';
    $h .= '<th>完了日</th>';
    $h .= '</tr>';

    foreach($tickets as $ticket){
        $h .= '<tr>';
        $h .= '<td>';
        $h .= '<a href="index.php?a=tie&amp;id=' . h($ticket['id']) . '">';
        $h .= h($ticket['id']) . '</a></td>';
        $h .= '<td>';
        $h .= '<a href="index.php?a=tv&amp;id=' . h($ticket['task_id']) . '">';
        $h .= h($ticket['task_id']) . '</a></td>';
        $h .= '<td>' . h($ticket['person']) . '</td>';
        $h .= '<td>' . h($ticket['task_applicant']) . '</td>';
        $h .= '<td>' . convert_text_to_html(make_summary($ticket['task_content'], 40)) . '</td>';
        $h .= '<td>' . convert_text_to_html(make_summary($ticket['content'], 40)) . '</td>';
        $h .= '<td>' . h($ticket['estimate_time']) . '</td>';
        $h .= '<td>' . h($ticket['results_time']) . '</td>';
        $h .= '<td>' . h($ticket['completion_date']) . '</td>';
        $h .= '</tr>';
    }

    $h .= '</table>';

    return $h;
}

/// DB関連の関数群

function db_query($dbh, $sql, $params = array())
{
    $sth = $dbh->prepare($sql);
    if($sth === false){
        return false;
    }

    $res = $sth->execute($params);
    if($res === false){
        return false;
    }

    return $sth;
}

function db_create_schema($dbh)
{
    $sql = <<<EOS
CREATE TABLE tasks(
id                       INTEGER,
revision_id              INTEGER DEFAULT 0,
accept_date              INTEGER,
accept_person            TEXT,
applicant                TEXT,
content                  TEXT,
deleted                  INTEGER DEFAULT 0,
update_date              INTEGER,
remote_addr              TEXT,
PRIMARY KEY(id, revision_id)
)
EOS;

    $res = db_query($dbh, $sql);
    if(!$res){
        // print_r($dbh->errorInfo());
    }

    $sql = <<<EOS
CREATE TABLE tickets(
id                       INTEGER,
revision_id              INTEGER DEFAULT 0,
task_id                  INTEGER,
person                   TEXT,
content                  TEXT,
estimate_time            REAL,
results_time             REAL,
completion_date          INTEGER,
deleted                  INTEGER DEFAULT 0,
update_date              INTEGER,
remote_addr              TEXT,
PRIMARY KEY(id, revision_id)
)
EOS;

    $res = db_query($dbh, $sql);
    if(!$res){
        // print_r($dbh->errorInfo());
    }

    $sql = <<<EOS
CREATE VIEW valid_tasks AS
SELECT
 *
FROM
 (
 SELECT
  t1.id AS id,
  MAX(t2.revision_id) AS revision_id
 FROM
  tasks t1
  LEFT JOIN tasks t2
  ON (t1.id=t2.id)
 GROUP BY
  t1.id
 ) AS t3
 LEFT JOIN tasks t
 ON t.id=t3.id AND t.revision_id=t3.revision_id

WHERE
 deleted=0
EOS;

    $res = db_query($dbh, $sql);
    if(!$res){
        // print_r($dbh->errorInfo());
    }

    $sql = <<<EOS
CREATE VIEW valid_tickets AS
SELECT
 t.*
FROM
 (
 SELECT
  t1.id AS id,
  MAX(t2.revision_id) AS revision_id
 FROM
  tickets t1
  LEFT JOIN tickets t2
  ON (t1.id=t2.id)
 GROUP BY
  t1.id
 ) AS t3
 LEFT JOIN tickets t
  ON t.id=t3.id AND t.revision_id=t3.revision_id
 JOIN valid_tasks tasks
  ON t.task_id=tasks.id

WHERE
 t.deleted=0 AND tasks.deleted=0
EOS;

    $res = db_query($dbh, $sql);
    if(!$res){
        // print_r($dbh->errorInfo());
    }

    $sql = <<<EOS
CREATE INDEX tickets_task_id_index ON tickets(task_id)
EOS;

    $res = db_query($dbh, $sql);
    if(!$res){
        // print_r($dbh->errorInfo());
    }
}

function db_add_task($dbh, $task)
{
    $sql = <<<EOS
INSERT INTO tasks
 (
 id,
 accept_date,
 accept_person,
 applicant,
 content,
 update_date,
 remote_addr) 
VALUES
 (
 (SELECT CASE WHEN MAX(id) IS NULL THEN 0 ELSE MAX(id) END FROM tasks) + 1,
 :accept_date,
 :accept_person,
 :applicant,
 :content,
 :update_date,
 :remote_addr
 )
EOS;

    $params = array(':accept_date' => $task['accept_date'],
                    ':accept_person' => $task['accept_person'],
                    ':applicant' => $task['applicant'],
                    ':content' => $task['content'],
                    ':update_date' => time(),
                    ':remote_addr' => $_SERVER['REMOTE_ADDR']);

    $res = db_query($dbh, $sql, $params);
    if($res === false){
        print_r( $dbh->errorInfo() );
        return false;
    }

    $sql = <<<EOS
SELECT CASE WHEN MAX(id) IS NULL THEN 0 ELSE MAX(id) END FROM tasks
EOS;

    $sth = db_query($dbh, $sql);
    if($sth === false){
        print_r( $dbh->errorInfo() );
        return false;
    }

    $res = $sth->fetch(PDO::FETCH_NUM);
    if($res === false){
        return false;
    }

    return $res[0];
}

function db_update_task($dbh, $task)
{
    $sql = <<<EOS
INSERT INTO tasks
(
 id,
 revision_id,
 accept_date,
 accept_person,
 applicant,
 content,
 update_date,
 remote_addr
) 
VALUES (
 :id,
 (SELECT MAX(revision_id) + 1 FROM tasks WHERE id=:id),
 :accept_date,
 :accept_person,
 :applicant,
 :content,
 :update_date,
 :remote_addr
)
EOS;

    $params = array(':accept_date' => $task['accept_date'],
                    ':accept_person' => $task['accept_person'],
                    ':applicant' => $task['applicant'],
                    ':content' => $task['content'],
                    ':update_date' => time(),
                    ':id' => $task['id'],
                    ':remote_addr' => $_SERVER['REMOTE_ADDR']);

    $sth = db_query($dbh, $sql, $params);
    if($sth === false){
        print_r( $dbh->errorInfo() );
        return false;
    }

    return true;
}

function db_delete_task($dbh, $id)
{
    $sql = <<<EOS
INSERT INTO tasks(
 deleted,
 id,
 revision_id,
 accept_date,
 accept_person,
 applicant,
 content,
 update_date,
 remote_addr
)
SELECT
 1,
 id,
 (SELECT MAX(revision_id) + 1 FROM tasks WHERE id=:id),
 accept_date,
 accept_person,
 applicant,
 content,
 :update_date,
 :remote_addr
FROM
 tasks
WHERE
 id=:id AND revision_id = (SELECT MAX(revision_id) FROM tasks WHERE id=:id)
EOS;

    $params = array(
        'id' => $id,
        'update_date' => time(),
        'remote_addr' => $_SERVER['REMOTE_ADDR']
        );

    $sth = db_query($dbh, $sql, $params);
    if($sth === false){
        print_r($dbh->errorInfo());
        return false;
    }

    return true;
}

function db_get_task($dbh, $id)
{
    $task = '';

    $sql = 'SELECT * FROM valid_tasks WHERE id=:id';
    $params = array(':id' => $id);
    $sth = db_query($dbh, $sql, $params);
    if($sth === false){
        return false;
    }

    $task = $sth->fetch();

    return $task;
}

function db_search_task_query($query)
{
    $q = array();
    if($query['text'] != ''){
        $q[] = <<<EOS
(tasks.applicant LIKE :query
 OR tasks.content LIKE :query
 OR tickets.person LIKE :query
 OR tickets.content LIKE :query )
EOS;
    }

    if($query['person'] != ''){
        $q[] = <<<EOS
(tasks.accept_person = :person
 OR tickets.person = :person) 
EOS;
    }

    if($query['date1'] != ''){
        $q[] = <<<EOS
(tasks.accept_date >= :date1
 OR tickets.completion_date >= :date1) 
EOS;
    }

    if($query['date2'] != ''){
        $q[] = <<<EOS
(tasks.accept_date <= :date2
 OR tickets.completion_date <= :date2) 
EOS;
    }

    $sql = '';
    if(count($q) > 0){
        $sql = ' WHERE ';
    }

    $sql .= implode(' AND ', $q);

    return $sql;
}

function db_search_task($dbh, $query, $limit, $offset, $order)
{
    $tasks = array();

    $sub_sql = ' (SELECT DISTINCT tasks.* FROM valid_tasks AS tasks LEFT JOIN valid_tickets AS tickets ON tasks.id=tickets.task_id ' . db_search_task_query($query) .  ') ';

    $sql = <<<EOS
SELECT tasks.*
 , CASE
 WHEN COUNT(tickets.id) = 0
 THEN tasks.update_date
 ELSE
  CASE
  WHEN (tasks.update_date > MAX(tickets.update_date)) 
  THEN tasks.update_date
  ELSE MAX(tickets.update_date) 
  END
 END update_date2
 , SUM(CASE WHEN tickets.id IS NOT NULL THEN 1 ELSE 0 END) AS ticket_count
 , SUM(CASE WHEN tickets.completion_date = '' THEN 1 ELSE 0 END) AS incomplete_ticket_count
 , CASE
   WHEN (tasks.accept_date > MAX(tickets.completion_date)) 
   THEN tasks.accept_date
   ELSE MAX(tickets.completion_date)
   END completion_date2
 , SUM(tickets.results_time) AS total_results_time
 , SUM(CASE WHEN tickets.completion_date='' THEN tickets.estimate_time ELSE 0 END) incomplete_time
FROM
 {$sub_sql} AS tasks
 LEFT JOIN valid_tickets AS tickets
  ON tasks.id=tickets.task_id

EOS;
    
    $sql .= ' GROUP BY tasks.id ';
    
    if($query['only_incomplete']){
        $sql .= <<<EOS
HAVING
ticket_count = 0 OR incomplete_ticket_count > 0

EOS;
    }

    $sql .= <<<EOS
ORDER BY

EOS;

    if($order == 'completion_date'){
        $sql .= <<<EOS
completion_date2 DESC

EOS;
    }else{
        $sql .= <<<EOS
update_date2 DESC

EOS;
    }
    
    $sql .= <<<EOS
LIMIT :limit OFFSET:offset 
EOS;

    // echo '<pre>' . $sql . '</pre>';

    $params = array(':limit' => $limit,
                    ':offset' => $offset);
    
    if($query['text'] != ''){
        $params[':query'] = '%' . $query['text'] . '%';
    }

    if($query['person'] != ''){
        $params[':person'] = $query['person'];
    }

    if($query['date1'] != ''){
        $params[':date1'] = $query['date1'];
    }

    if($query['date2'] != ''){
        $params[':date2'] = $query['date2'];
    }

    $sth = db_query($dbh, $sql, $params);
    if($sth === false){
        print_r($dbh->errorInfo());
        return false;
    }

    $res = $sth->fetchAll();

    if($res === false){
        return false;
    }

    $tasks = $res;

    return $tasks;
}

function db_statistics_task($dbh, $query)
{
    $params = array();

    $sub_sql = '';
    $sub_sql2= '';
    
    if($query['person'] != '' || $query['date1'] != '' || $query['date2'] != ''){
        $q = array();
        if($query['person'] != ''){
            $q[] = ' tickets.person=:person ';
            $q2[] = ' tickets.person=:person ';
            $params[':person'] = $query['person'];
        }

        if($query['date1'] != ''){
            $q[] = ' tickets.completion_date >= :date1 ';
            $q2[] = ' tasks.accept_date >= :date1 ';
            $params[':date1'] = $query['date1'];
        }

        if($query['date2'] != ''){
            $q[] = ' tickets.completion_date <= :date2 ';
            $q2[] = ' tasks.accept_date <= :date2 ';
            $params[':date2'] = $query['date2'];
        }

        $sub_sql = implode(' AND ', $q);
        $sub_sql2 = implode(' AND ', $q2);
    }else{
        $sub_sql = ' 1 ';
        $sub_sql2 = ' 1 ';
    }
    
    $sub_sql3 = ' (SELECT DISTINCT tasks.* FROM valid_tasks AS tasks LEFT JOIN valid_tickets AS tickets ON tasks.id=tickets.task_id ' . db_search_task_query($query) .  ') ';

    $sql = <<<EOS
SELECT
 COUNT(*) AS task_count
 , SUM(ticket_count) AS ticket_count
 , SUM(incomplete_ticket_count) AS incomplete_ticket_count
 , SUM(total_results_time) AS total_results_time
 , SUM(incomplete_time) AS incomplete_time
 , SUM(total_results_time_2) AS total_results_time_2
 , SUM(incomplete_time_2) AS incomplete_time_2
FROM
(
SELECT
 SUM(CASE WHEN tickets.id IS NOT NULL THEN 1 ELSE 0 END) AS ticket_count
 , SUM(CASE WHEN tickets.completion_date = '' THEN 1 ELSE 0 END) AS incomplete_ticket_count
 , SUM(tickets.results_time) AS total_results_time
 , SUM(CASE WHEN tickets.completion_date='' THEN tickets.estimate_time ELSE 0 END) incomplete_time
 , SUM(CASE WHEN {$sub_sql} THEN tickets.results_time ELSE 0 END) AS total_results_time_2
 , SUM(CASE WHEN {$sub_sql2} AND tickets.completion_date='' THEN tickets.estimate_time ELSE 0 END) incomplete_time_2
FROM 
 {$sub_sql3} AS tasks
 LEFT JOIN valid_tickets AS tickets
  ON tasks.id=tickets.task_id

EOS;

    $sql .= db_search_task_query($query);

    $sql .= ' GROUP BY tasks.id ';

    if($query['only_incomplete']){
        $sql .= '

 HAVING ';

        $sql .= '
ticket_count = 0 OR incomplete_ticket_count > 0
';
    }

    $sql .= '
)
';
    if($query['text'] != ''){
        $params['query'] = '%' . $query['text'] . '%';
    }

    // echo '<pre>' . $sql . '</pre>';

    $sth = db_query($dbh, $sql, $params);
    if($sth === false){
        print_r($dbh->errorInfo());
        return false;
    }

    $res = $sth->fetch(PDO::FETCH_ASSOC);
    if($res === false){
        print_r($dbh->errorInfo());
        return false;
    }

    // print_r($res);

    return $res;
}

function db_search_ticket($dbh, $task_id, $only_incomplete = false)
{
    $tickets = array();

    $sql = <<<EOS
SELECT
 tickets.*,
 tasks.applicant AS task_applicant,
 tasks.content AS task_content,
 CASE WHEN completion_date = '' THEN 0
 ELSE 1
 END completed
FROM
 valid_tickets AS tickets
 LEFT JOIN valid_tasks AS tasks
 ON tickets.task_id=tasks.id

EOS;

    if(!is_null($task_id)){
        $sql .= <<<EOS
WHERE tickets.task_id=:task_id

EOS;
    }

    if($only_incomplete){
        if(!is_null($task_id)){
            $sql .= ' AND ';
        }else{
            $sql .= ' WHERE ';
        }
        
        $sql .= <<<EOS
completion_date=''
ORDER BY
 tickets.person,
 tickets.update_date DESC,
 tickets.id

EOS;
    }else{
        $sql .= <<<EOS
ORDER BY
 completed DESC,
 completion_date,
 tickets.id

EOS;
    }

    $params = array();
    if(!is_null($task_id)){
        $params[':task_id'] = $task_id;
    }

    $sth = db_query($dbh, $sql, $params);
    if($sth === false){
        print_r( $dbh->errorInfo() );
        return false;
    }

    $res = $sth->fetchAll();

    if($res === false){
        print_r( $dbh->errorInfo() );
        return false;
    }

    $tickets = $res;

    return $tickets;
}

function db_add_ticket($dbh, $ticket)
{
    $sql = <<<EOS
INSERT INTO tickets(
 id,
 task_id,
 person,
 content,
 estimate_time,
 results_time,
 completion_date,
 update_date)
VALUES (
 (SELECT CASE WHEN MAX(id) IS NULL THEN 0 ELSE MAX(id) END FROM tickets) + 1,
 :task_id,
 :person,
 :content,
 :estimate_time,
 :results_time,
 :completion_date,
 :update_date)
EOS;

    $sth = $dbh->prepare($sql);
    if($sth === false){
        print_r( $dbh->errorInfo() );
        return false;
    }

    $params = array(':task_id' => $ticket['task_id'],
                    ':person' => $ticket['person'],
                    ':content' => $ticket['content'],
                    ':estimate_time' => $ticket['estimate_time'],
                    ':results_time' => $ticket['results_time'],
                    ':completion_date' => $ticket['completion_date'],
                    ':update_date' => time());

    $sth = db_query($dbh, $sql, $params);
    if($sth === false){
        print_r( $dbh->errorInfo() );
        return false;
    }

    $sql = <<<EOS
SELECT CASE WHEN MAX(id) IS NULL THEN 0 ELSE MAX(id) END FROM tickets
EOS;

    $sth = db_query($dbh, $sql);
    if($sth === false){
        print_r( $dbh->errorInfo() );
        return false;
    }

    $res = $sth->fetch(PDO::FETCH_NUM);
    if($res === false){
        return false;
    }

    return $res[0];
}

function db_get_ticket($dbh, $id)
{
    $ticket = '';

    $sql = 'SELECT * FROM valid_tickets WHERE id=:id';
    $params = array(':id' => $id);
    $sth = db_query($dbh, $sql, $params);
    if($sth === false){
        return false;
    }

    $ticket = $sth->fetch();

    return $ticket;
}

function db_update_ticket($dbh, $ticket)
{
    $sql = <<<EOS
INSERT INTO tickets(
 id,
 revision_id,
 task_id,
 person,
 content,
 estimate_time,
 results_time,
 completion_date,
 update_date)
VALUES (
 :id,
 (SELECT MAX(revision_id) + 1 FROM tickets WHERE id=:id),
 :task_id,
 :person,
 :content,
 :estimate_time,
 :results_time,
 :completion_date,
 :update_date)
EOS;

    $params = array(':person' => $ticket['person'],
                    ':task_id' => $ticket['task_id'],
                    ':content' => $ticket['content'],
                    ':estimate_time' => $ticket['estimate_time'],
                    ':results_time' => $ticket['results_time'],
                    ':completion_date' => $ticket['completion_date'],
                    ':update_date' => time(),
                    ':id' => $ticket['id']);

    $sth = db_query($dbh, $sql, $params);
    if($sth === false){
        return false;
    }

    return true;
}

function db_delete_ticket($dbh, $id)
{
    $sql = <<<EOS
INSERT INTO tickets(
 deleted,
 id,
 revision_id,
 task_id,
 person,
 content,
 estimate_time,
 results_time,
 completion_date,
 update_date,
 remote_addr
 )
SELECT
 1,
 :id,
 (SELECT MAX(revision_id) + 1 FROM tickets WHERE id=:id),
 task_id,
 person,
 content,
 estimate_time,
 results_time,
 completion_date,
 :update_date,
 :remote_addr
FROM
 tickets
WHERE
 id=:id AND revision_id=(SELECT MAX(revision_id) FROM tickets WHERE id=:id)
EOS;

    $params = array(
        ':id' => $id,
        ':update_date' => time(),
        ':remote_addr' => $_SERVER['REMOTE_ADDR']
        );

    $sth = db_query($dbh, $sql, $params);
    if($sth === false){
        return false;
    }

    return true;
}

/// セッション関数

function session_token_init()
{
    if(!isset($_SESSION['tokens']) || !is_array($_SESSION['tokens'])){
        $_SESSION['tokens'] = array();
    }
}

function session_token_push($token)
{
    session_token_init();
    
    array_push($_SESSION['tokens'], $token);

    array_unique($_SESSION['tokens']);

    if(count($_SESSION['tokens']) > 5){
        array_shift($_SESSION['tokens']);
    }
}

function session_token_exists($token)
{
    session_token_init();

    return in_array($token, $_SESSION['tokens']);
}

function session_token_remove($token)
{
    session_token_init();

    for($i = 0; $i < count($_SESSION['tokens']); $i++){
        if($_SESSION['tokens'][$i] == $token){
            array_splice($_SESSION['tokens'], $i, 1);
            return;
        }
    }
}

/// その他の関数

function convert_values_to_select_options($values)
{
    $options = array();
    foreach($values as $v){
        $options[] = array('value' => $v, 'text' => $v);
    }

    return $options;
}

function token()
{
    return md5(uniqid(mt_rand(), true));
}

function make_rss($items)
{
    $x = '';
    
    $url = get_url();

    $x .= '<?xml version="1.0" encoding="utf-8" ?>';
    $x .= '<rdf:RDF 
  xmlns="http://purl.org/rss/1.0/"
  xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#"
  xmlns:dc="http://purl.org/dc/elements/1.1/"
  xml:lang="ja">';
    $x .= '<channel rdf:about="http://www.kanzaki.com/info/rss.rdf">
  <title>タスク管理</title>
  <link>' . $url . '</link>
  <description></description>';
    $x .= '<items><rdf:Seq>';
    $x .= "\n";
    foreach($items as $i){
        $x .= '<rdf:li rdf:resource="';
        $x .= $i['link'];
        $x .= '" />';
        $x .= "\n";
    }
    
    $x .= '</rdf:Seq></items>';
    $x .= '</channel>';
    $x .= "\n";

    foreach($items as $i){
        $x .= '<item rdf:about="' . $i['link'] . '">';
        $x .= '<title>' . $i['title'] . '</title>';
        $x .= '<link>' . $i['link'] . '</link>';
        $x .= '<description><![CDATA[' . $i['description'] . ']]></description>';
        $x .= '<dc:date>' . date('Y-m-d\TH:i:s+09:00', $i['date']) . '</dc:date>';
        $x .= '</item>';
        $x .= "\n";
    }

    $x .= '</rdf:RDF>';

    return $x;
}

function get_url()
{
    return 'http://' . $_SERVER['SERVER_NAME'] . $_SERVER['SCRIPT_NAME'];
}

function is_valid_date_string($s)
{
    if($s === ''){
        return true;
    }

    if(!preg_match('/^[0-9]{4}\/[0-9]{2}\/[0-9]{2}$/', $s)){
        return false;
    }

    list($y, $m, $d) = explode('/', $s);

    if(!checkdate($m, $d, $y)){
        return false;
    }
    
    return true;
}

function is_valid_time_string($s)
{
    if($s === ''){
        return true;
    }

    if(!preg_match('/^\d+(?:\.\d+)?$/', $s)){
        return false;
    }

    $time = (double)$s;

    if(($time != 0) && (fmod($time, 0.25) != 0)){
        return false;
    }

    return true;
}

/// ここから処理スタート

session_start();

$dbh = new PDO($dsn);

db_create_schema($dbh);

$action = isset($_GET['a']) ? $_GET['a'] : null;

if($action == 'te'){
    // タスク編集画面を表示
    $id = isset($_GET['id']) ? $_GET['id'] : null;
    $task = db_get_task($dbh, $id);
    if($task === false){

    }

    $tickets = db_search_ticket($dbh, $id);
    if($tickets === false){
    }

    $task_token = token();
    session_token_push($task_token);

    $ticket_token = token();
    session_token_push($ticket_token);

    $h = '';
    $h .= '<p><a href="index.php?a=tv&amp;id=' . h($task['id']) . '">編集を終了します</a> | ';
    $h .= '<a href="index.php?a=tdc&amp;id=' . h($task['id']) . '">このタスクを削除します</a></p>';
    $h .= make_html_task_edit($task, $task_token);

    echo make_html('タスク編集', $h);

}elseif($action == 'tv'){
    // タスク画面を表示
    $id = isset($_GET['id']) ? $_GET['id'] : null;
    $task = db_get_task($dbh, $id);
    if($task === false){

    }

    $tickets = db_search_ticket($dbh, $id);
    if($tickets === false){
    }

    $task_token = token();
    session_token_push($task_token);

    $ticket_token = token();
    session_token_push($ticket_token);

    $h = '';
    $h .= '<p><a href="index.php?a=te&amp;id=' . h($task['id']) . '">このタスクを編集します</a> | ';
    $h .= '<a href="index.php?a=tdc&amp;id=' . h($task['id']) . '">このタスクを削除します</a></p>';
    $h .= make_html_task_edit($task, $task_token, true);

    $ticket = array(
        'id' => null,
        'task_id' => $id,
        'person' => isset($_COOKIE['accept_person']) ? $_COOKIE['accept_person'] : '',
        'content' => '',
        'estimate_time' => '',
        'results_time' => '',
        'completion_date' => '');

    $h .= make_html_ticket_list($tickets);
    $h .= make_html_ticket_edit($ticket, $ticket_token);

    echo make_html('タスク表示', $h);

}elseif($action == 'tn'){
    // タスク追加画面を表示

    $task = array(
        'id' => null,
        'accept_date' => date('Y/m/d'), 
        'accept_person' => isset($_COOKIE['accept_person']) ? $_COOKIE['accept_person'] : '',
        'applicant' => '',
        'content' => '');

    $token = token();

    session_token_push($token);

    $h = make_html_task_edit($task, $token);

    echo make_html('タスク追加', $h);

}elseif($action == 'ta'){
    // タスク追加
    $task = array(
        'accept_date' => $_POST['accept_date'],
        'accept_person' => normalize_kana($_POST['accept_person']),
        'applicant' => normalize_kana($_POST['applicant']),
        'content' => normalize_kana($_POST['content']));

    // 入力値検査
    if(!is_valid_date_string($task['accept_date'])){
        $h = '<p>日付が正しくありません。</p>';
        echo make_html('タスク追加', $h);
        exit;
    }

    setcookie('accept_person', $_POST['accept_person'], time() + 60 * 60 * 24 * 7);

    $token = $_POST['token'];

    $h = '';
    if(!session_token_exists($token)){
        $h .= '<p>二重投稿を防止しました</p>';
    }else{
        session_token_remove($token);

        $res = db_add_task($dbh, $task);
        if($res !== false){
            header('Location: index.php?a=tv&id=' . ue($res));
            exit;
        }

        $h .= '<p>追加できませんでした</p>';
    }

    echo make_html('タスク追加', $h);
    
}elseif($action == 'tu'){
    // タスク更新
    $task = array(
        'id' => $_POST['id'],
        'accept_date' => $_POST['accept_date'],
        'accept_person' => normalize_kana($_POST['accept_person']),
        'applicant' => normalize_kana($_POST['applicant']),
        'content' => normalize_kana($_POST['content']));

    // 入力値検査
    if(!is_valid_date_string($task['accept_date'])){
        $h = '<p>日付が正しくありません。</p>';
        echo make_html('タスク追加', $h);
        exit;
    }

    $token = $_POST['token'];

    $h = '';
 
    if(!session_token_exists($token)){
        $h .= '<p>二重投稿を防止しました</p>';
    }else{
        session_token_remove($token);

        // old task
        $ot = db_get_task($dbh, $task['id']);
        if($ot === false){
            // 更新元タスクの取得に失敗
            $h .= '<p>タスクが見つかりません</p>';
        }else if(($ot['accept_date'] == $task['accept_date']
                  && $ot['accept_person'] == $task['accept_person']
                  && $ot['applicant'] == $task['applicant']
                  && $ot['content'] == $task['content'])
                 || db_update_task($dbh, $task) !== false)
        {
            // 更新なし、または更新成功
            header('Location: index.php?a=tv&id=' . ue($task['id']));
            exit;
        }

        $h .= '<p>タスク更新失敗</p>';
    }
    
    $h .= '<p><a href="index.php?a=te&amp;id=' . h($task['id']) . '">タスク確認</a></p>';
    
    echo make_html('タスク更新', $h);

}elseif($action == 'tdc'){
    // タスク削除確認
    $id = isset($_GET['id']) ? $_GET['id'] : null;

    $token = token();
    session_token_push($token);

    $h = '<p><a href="index.php?a=td&amp;id=';
    $h .= h($id);
    $h .= '&amp;token=' . $token;
    $h .= '">削除します</a></p>';
    $h .= '<p><a href="index.php?a=te&amp;id=';
    $h .= h($id);
    $h .= '">削除しません</a></p>';

    echo make_html('タスク削除確認', $h);

}elseif($action == 'td'){
    // タスク削除
    $id = isset($_GET['id']) ? $_GET['id'] : null;
    $token = $_GET['token'];

    $h = '';
    if(!session_token_exists($token)){
        $h .= '<p>二重投稿を防止しました</p>';
    }else{
        session_token_remove($token);
        
        $res = db_delete_task($dbh, $id);
        if($res !== false){
            header('Location: index.php');
            exit;;
        }
        
        $h .= '<p>タスク削除失敗</p>';
    }

    echo make_html('タスク削除', $h);

}elseif($action == 'tia'){
    // チケット追加
    $ticket = array(
        'task_id' => $_POST['task_id'],
        'person' => normalize_kana($_POST['person']),
        'content' => normalize_kana($_POST['content']),
        'estimate_time' => $_POST['estimate_time'],
        'results_time' => $_POST['results_time'],
        'completion_date' => $_POST['completion_date']);

    // 入力値検査
    if(!is_valid_date_string($ticket['completion_date'])){
        $h = '<p>日付が正しくありません。</p>';
        echo make_html('チケット追加', $h);
        exit;
    }

    if(!is_valid_time_string($ticket['estimate_time'])){
        $h = '<p>予想時間が正しくありません。</p>';
        echo make_html('チケット追加', $h);
        exit;
    }

    if(!is_valid_time_string($ticket['results_time'])){
        $h = '<p>実績時間が正しくありません。</p>';
        echo make_html('チケット追加', $h);
        exit;
    }

    $token = $_POST['token'];

    $h = '';
    if(!session_token_exists($token)){
        $h .= '<p>二重投稿を防止しました</p>';
    }else{
        session_token_remove($token);
        
        $res = db_add_ticket($dbh, $ticket);
        if($res !== false){
            header('Location: index.php?a=tv&id=' . ue($ticket['task_id']));
            exit;
        }
        
        $h .= '<p>チケット追加失敗</p>';
    }

    $h .= '<p><a href="index.php?a=tv&amp;id=' . $ticket['task_id'] . '">タスク確認</a></p>';

    echo make_html('チケット追加', $h);

}elseif($action == 'tie'){
    // チケット編集
    $id = isset($_GET['id']) ? $_GET['id'] : null;

    $token = token();
    session_token_push($token);
    
    $ticket = db_get_ticket($dbh, $id);

    $h = '<p><a href="index.php?a=tidc&amp;id=' . h($ticket['id']) . '">このチケットを削除する</a></p>';
    $h .= make_html_ticket_edit($ticket, $token);

    echo make_html('チケット編集', $h);

}elseif($action == 'tiu'){
    // チケット更新
    $id = $_POST['id'];
    
    $ticket['person'] = normalize_kana($_POST['person']);
    $ticket['content'] = normalize_kana($_POST['content']);
    $ticket['estimate_time'] = $_POST['estimate_time'];
    $ticket['results_time'] = $_POST['results_time'];
    $ticket['completion_date'] = $_POST['completion_date'];

    // 入力値検査
    if(!is_valid_date_string($ticket['completion_date'])){
        $h = '<p>日付が正しくありません。</p>';
        echo make_html('タスク追加', $h);
        exit;
    }

    if(!is_valid_time_string($ticket['estimate_time'])){
        $h = '<p>予想時間が正しくありません。</p>';
        echo make_html('チケット追加', $h);
        exit;
    }

    if(!is_valid_time_string($ticket['results_time'])){
        $h = '<p>実績時間が正しくありません。</p>';
        echo make_html('チケット追加', $h);
        exit;
    }

    $token = $_POST['token'];

    $h = '';
    if(!session_token_exists($token)){
        $h .= '<p>二重投稿を防止しました</p>';
    }else{
        session_token_remove($token);

        $ot = db_get_ticket($dbh, $id);
        $ticket['id'] = $ot['id'];
        $ticket['task_id'] = $ot['task_id'];

        if($ot === false){
            // 更新元チケットの取得に失敗
            $h .= '<p>チケットが見つかりません</p>';
        }else if($ot['person'] === $ticket['person']
                 && $ot['content'] == $ticket['content']
                 && $ot['estimate_time'] == $ticket['estimate_time']
                 && $ot['results_time'] == $ticket['results_time']
                 && $ot['completion_date'] == $ticket['completion_date']
                 || db_update_ticket($dbh, $ticket) !== false)
        {
            // 更新なし、または更新成功
            header('Location: index.php?a=tv&id=' . ue($ticket['task_id']));
            exit;
        }
        
        $h .= '<p>チケット更新失敗</p>';
    }
    
    $h .= '<p><a href="index.php?a=tv&amp;id=' . h($ticket['task_id']) . '">タスクへ</a></p>';

    $h .= '<p><a href="index.php?a=tie&amp;id=' . h($ticket['id']) . '">チケット確認</a></p>';

    echo make_html('チケット更新', $h);

}elseif($action == 'tidc'){
    // チケット削除確認
    $id = isset($_GET['id']) ? $_GET['id'] : null;

    $token = token();
    session_token_push($token);
    
    $h = '<p><a href="index.php?a=tid&amp;id=';
    $h .= h($id);
    $h .= '&amp;token=' . h($token);
    $h .= '">削除します</a></p>';
    $h .= '<p><a href="index.php?a=tie&amp;id=';
    $h .= h($id);
    $h .= '">削除しません</a></p>';

    echo make_html('チケット削除確認', $h);

}elseif($action == 'tid'){
    // TIcket Delete
    $id = isset($_GET['id']) ? $_GET['id'] : null;
    $token = $_GET['token'];

    $h = '';
    if(!session_token_exists($token)){
        $h .= '<p>二重投稿を防止しました</p>';
    }else{
        session_token_remove($token);

        $ticket = db_get_ticket($dbh, $id);
        
        $res = db_delete_ticket($dbh, $id);
        if($res !== false){
            header('Location: index.php?a=tv&id=' . ue($ticket['task_id']));
            exit;
        }
        
        $h .= '<p>失敗しました</p>';

        $h .= '<p><a href="index.php?a=tv&amp;id=';
        $h .= h($ticket['task_id']);
        $h .= '">タスクへ</a></p>';
    }

    echo make_html('チケット削除', $h);

}elseif($action == 'todo'){
    // ToDo

    $tickets = db_search_ticket($dbh, null, true);

    $h = make_html_todo($tickets);

    echo make_html('チケット一覧', $h);

}elseif($action == 'rss'){
    // RSS
    
    $query['text'] = '';
    $query['only_incomplete'] = false;
    $query['person'] = '';
    $query['date1'] = '';
    $query['date2'] = '';

    $tasks = db_search_task($dbh, $query, 20, 0, 'update_date');
    if($tasks === false){
        $tasks = array();
    }

    $items = array();
    foreach($tasks as $t){
        $i = array();
        $i['title'] = $t['id'] . ' ' . mb_strimwidth($t['content'], 0, 25, '...');
        $i['link'] = get_url() . '?a=tv&amp;id=' . $t['id'];
        $i['description'] = $t['content'];
        $i['date'] = $t['update_date2'];

        $items[] = $i;
    }

    echo make_rss($items);

}elseif($action == 'search'){
    // タスク一覧画面 
    $limit = 20;

    $offset = isset($_GET['of']) ? $_GET['of'] : 0;

    $query['text'] = isset($_GET['q']) ? $_GET['q'] : '';
    $query['only_incomplete'] = isset($_GET['oi']) ? true : false;
    $query['person'] = isset($_GET['p']) ? $_GET['p'] : '';
    $query['date1'] = isset($_GET['d1']) ? $_GET['d1'] : '';
    $query['date2'] = isset($_GET['d2']) ? $_GET['d2'] : '';

    $order = isset($_GET['order']) ? $_GET['order'] : '';

    $stat = db_statistics_task($dbh, $query);

    $tasks = db_search_task($dbh, $query, $limit, $offset, $order);
    $total = $stat['task_count'];

    if($tasks === false){
        echo make_html('タスク一覧', '一覧の取得に失敗しました');
        exit;
    }

    $h = '';
    $h .= make_html_task_search_form($query);

    $total_results_time_2 = h($stat['total_results_time_2']);
    $incomplete_time_2 = h($stat['incomplete_time_2']);
    $total_results_time = h($stat['total_results_time']);
    $incomplete_time = h($stat['incomplete_time']);

    $h .= <<<EOS
タスク単位の集計： 実績 {$total_results_time} 未完 {$incomplete_time}
/ チケット単位の集計： 実績 {$total_results_time_2} 未完 {$incomplete_time_2}
EOS;

    $h .= '<p>' . h((int)$total) . ' 件中 ' . ($offset + 1) . ' - ' . ($offset + count($tasks)) .  ' 件目表示</p>';
    $h .= make_html_task_list_navigation($query, $limit, $offset, $total, $order);
    $h .= make_html_task_list($tasks);

    echo make_html('検索', $h);
}else{
    $h = '';
    $h .= '<h1>ダッシュボード</h1>';

    $query['text'] = '';
    $query['only_incomplete'] = true;
    $query['person'] = '';
    $query['date1'] = '';
    $query['date2'] = '';

    $stat = db_statistics_task($dbh, $query);

    $incomplete_tasks = db_search_task($dbh, $query, 5, 0, 'update_date');

    $query['only_incomplete'] = false;
    $all_tasks = db_search_task($dbh, $query, 5, 0, 'update_date');

    $h .= '<p><strong>' . $stat['task_count'] . '</strong> 個 <strong>' . $stat['incomplete_time'] . '</strong> 時間の未処理があります。最初の5個を表示しています。 (<a href="index.php?a=search&amp;oi">全て表示</a>)</p>';

    $h .= make_html_task_list($incomplete_tasks, true);

    $h .= '<p>直近の5タスクを表示しています。(<a href="index.php?a=search">全て表示</a>)</p>';
    $h .= make_html_task_list($all_tasks, true);

    echo make_html('ダッシュボード', $h);
}

